dangerous_iam = [
    {
        "Description": "This is a list of IAM actions that can be used to escalate privileges or move laterally.",
        "Name": "DangerousIAMActions",
        "ActionsNeeded": [
            "iam:PutGroupPolicy",
            "iam:PutRolePolicy",
            "iam:PutUserPolicy",
            "iam:AttachGroupPolicy",
            "iam:AttachRolePolicy",
            "iam:AttachUserPolicy",
            "iam:CreatePolicyVersion",
            "iam:SetDefaultPolicyVersion",
            "iam:AddUserToGroup",
            "iam:CreateLoginProfile",
            "iam:UpdateLoginProfile",
            "iam:CreateAccessKey",
            "iam:CreateRole",
            "sts:assumerole",
        ],
        "AllOrNone": False,
    }
]

write_actions = [
    {
        "Description": "These are all actions which may allow some kind of write privilege.",
        "Name": "WriteActions",
        "ActionsNeeded": [
            "*:Put*",
            "*:Create*",
            "*:Delete*",
            "*:Modify*",
            "*:Update*",
            "*:Attach*",
            "*:Detach*",
            "*:Associate*",
            "*:Disassociate*",
            "*:Add*",
            "*:Remove*",
            "*:Set*",
            "*:Enable*",
            "*:Disable*",
            "*:Reset*",
            "*:Stop*",
            "*:Terminate*",
            "*:Reboot*",
            "*:Start*",
        ],
        "AllOrNone": False,
    }
]


privescs = [
    {
        "Description": "",
        "Name": "CreateNewPolicyVersion",
        "ActionsNeeded": [
            "iam:CreatePolicyVersion",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "SetExistingDefaultPolicyVersion",
        "ActionsNeeded": [
            "iam:SetDefaultPolicyVersion",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "CreateEC2WithExistingIP",
        "ActionsNeeded": [
            "iam:PassRole",
            "ec2:RunInstances",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "CreateAccessKey",
        "ActionsNeeded": [
            "iam:CreateAccessKey",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "CreateLoginProfile",
        "ActionsNeeded": [
            "iam:CreateLoginProfile",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "UpdateLoginProfile",
        "ActionsNeeded": [
            "iam:UpdateLoginProfile",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "AttachRolePolicy",
        "ActionsNeeded": [
            "iam:AttachRolePolicy",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "PutRolePolicy",
        "ActionsNeeded": [
            "iam:PutRolePolicy",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "UpdateRolePolicyToAssumeIt",
        "ActionsNeeded": [
            "iam:UpdateAssumeRolePolicy",
            "sts:AssumeRole",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "PassExistingRoleToNewLambdaThenInvoke",
        "ActionsNeeded": [
            "iam:PassRole",
            "lambda:CreateFunction",
            "lambda:InvokeFunction",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "PassExistingRoleToNewLambdaThenInvokeCrossAccount",
        "ActionsNeeded": [
            "iam:PassRole",
            "lambda:CreateFunction",
            "lambda:AddPermission",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "PassExistingRoleToNewLambdaThenTriggerWithNewDynamo",
        "ActionsNeeded": [
            "iam:PassRole",
            "lambda:CreateFunction",
            "lambda:CreateEventSourceMapping",
            "dynamodb:CreateTable",
            "dynamodb:PutItem",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "PassExistingRoleToNewLambdaThenTriggerWithExistingDynamo",
        "ActionsNeeded": [
            "iam:PassRole",
            "lambda:CreateFunction",
            "lambda:CreateEventSourceMapping",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "PassExistingRoleToNewGlueDevEndpoint",
        "ActionsNeeded": [
            "iam:PassRole",
            "glue:CreateDevEndpoint",
            "glue:GetDevEndpoint",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "UpdateExistingGlueDevEndpoint",
        "ActionsNeeded": [
            "glue:UpdateDevEndpoint",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "PassExistingRoleToNewCloudFormation",
        "ActionsNeeded": [
            "iam:PassRole",
            "cloudformation:CreateStack",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "PassExistingRoleToNewDataPipeline",
        "ActionsNeeded": [
            "iam:PassRole",
            "datapipeline:CreatePipeline",
            "datapipeline:PutPipelineDefinition",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "EditExistingLambdaFunctionWithRole",
        "ActionsNeeded": [
            "lambda:UpdateFunctionCode",
        ],
        "AllOrNone": True,
    },
    {
        "Description": "",
        "Name": "PassExistingRoleToNewCodeStarProject",
        "ActionsNeeded": [
            "codestar:CreateProject",
            "iam:PassRole",
        ],
        "AllOrNone": True,
    },
]



import boto3
from botocore.config import Config
import os
import json
from concurrent.futures import ThreadPoolExecutor

from IAMActionHunter.lib.statement_parser import enumerate_actions_resources_for_statements


def get_all_iam_policies(target_aws_profile):
    """
    Get all IAM policies for roles and users for the specified AWS profile
    Args: target_aws_profile (str): AWS profile to use for authentication
    Returns: None
    """

    def ensure_list(item):
        """
        Ensure that the item is a list
        Args: item (object): item to check
        returns: list: item as a list
        """
        if isinstance(item, list):
            return item
        else:
            return [item]

    def get_role_policies(role):
        role_name = role["RoleName"]
        statements = []

        # Get statetments for inline policies attached to the role
        response = iam.list_role_policies(RoleName=role_name)
        for policy_name in response["PolicyNames"]:
            statement = iam.get_role_policy(RoleName=role_name, PolicyName=policy_name)["PolicyDocument"]["Statement"]
            statements += ensure_list(statement)

        # Get statements for managed policies attached to the role
        response = iam.list_attached_role_policies(RoleName=role_name)
        for policy in response["AttachedPolicies"]:
            policy_arn = policy["PolicyArn"]
            policy_name = policy_arn.split(":")[-1]
            version_id = iam.get_policy(PolicyArn=policy_arn)["Policy"]["DefaultVersionId"]
            statement = iam.get_policy_version(PolicyArn=policy_arn, VersionId=version_id)["PolicyVersion"]["Document"][
                "Statement"
            ]
            statements += ensure_list(statement)

        role_output = enumerate_actions_resources_for_statements(statements)

        # Save policies to file
        output_file = os.path.join(roles_output_dir, f"{role_name}.json")
        with open(output_file, "w") as f:
            json.dump(role_output, f)

    def get_user_policies(user):
        user_name = user["UserName"]
        statements = []

        # Get stetaments for inline policies attached to the user
        response = iam.list_user_policies(UserName=user_name)
        for policy_name in response["PolicyNames"]:
            statement = iam.get_user_policy(UserName=user_name, PolicyName=policy_name)["PolicyDocument"]["Statement"]
            statements += ensure_list(statement)

        # Get  statements for managed policies attached to the user
        response = iam.list_attached_user_policies(UserName=user_name)
        for policy in response["AttachedPolicies"]:
            policy_arn = policy["PolicyArn"]
            policy_name = policy_arn.split(":")[-1]
            version_id = iam.get_policy(PolicyArn=policy_arn)["Policy"]["DefaultVersionId"]
            statement = iam.get_policy_version(PolicyArn=policy_arn, VersionId=version_id)["PolicyVersion"]["Document"][
                "Statement"
            ]
            statements += ensure_list(statement)

        # Get statements for managed policies attached to the user's groups
        response = iam.list_groups_for_user(UserName=user_name)
        for group in response["Groups"]:
            group_name = group["GroupName"]
            response = iam.list_attached_group_policies(GroupName=group_name)
            for policy in response["AttachedPolicies"]:
                policy_arn = policy["PolicyArn"]
                policy_name = policy_arn.split(":")[-1]
                version_id = iam.get_policy(PolicyArn=policy_arn)["Policy"]["DefaultVersionId"]
                statement = iam.get_policy_version(PolicyArn=policy_arn, VersionId=version_id)["PolicyVersion"][
                    "Document"
                ]["Statement"]
                statements += ensure_list(statement)
        user_output = enumerate_actions_resources_for_statements(statements)

        # Save policies to file
        output_file = os.path.join(users_output_dir, f"{user_name}.json")
        with open(output_file, "w") as f:
            json.dump(user_output, f)

    # Use the specified profile for authentication
    session = boto3.Session(profile_name=target_aws_profile)

    # Create IAM client
    config = Config(retries=dict(max_attempts=10))
    iam = session.client("iam", config=config)

    # Get AWS account number
    sts = session.client("sts")
    account_id = sts.get_caller_identity().get("Account")

    print(f"Downloading IAM policies for account {account_id}...")

    # Create the output directory using the account number
    roles_output_dir = f"actionhunter_output/{account_id}/roles"
    users_output_dir = f"actionhunter_output/{account_id}/users"
    os.makedirs(roles_output_dir, exist_ok=True)
    os.makedirs(users_output_dir, exist_ok=True)

    # Paginate through all roles
    roles = []
    marker = None
    while True:
        if marker:
            response = iam.list_roles(Marker=marker)
        else:
            response = iam.list_roles()
        roles.extend(response["Roles"])
        if response["IsTruncated"]:
            marker = response["Marker"]
        else:
            break

    # Paginate through all users
    users = []
    marker = None
    while True:
        if marker:
            response = iam.list_users(Marker=marker)
        else:
            response = iam.list_users()
        users.extend(response["Users"])
        if response["IsTruncated"]:
            marker = response["Marker"]
        else:
            break

    with ThreadPoolExecutor(max_workers=10) as executor:
        # Save policies for all roles
        for role in roles:
            executor.submit(get_role_policies, role)

        # Save policies for all users
        for user in users:
            executor.submit(get_user_policies, user)

    print(f"Finished downloading IAM policies for account {account_id}.")
    print(f"Processing files for {account_id}...")



import json
import os
import argparse
import sys

import traceback

from policyuniverse.statement import Statement

from IAMActionHunter.lib.data_collection import get_all_iam_policies
from IAMActionHunter.lib.create_csv import process_json_and_append_to_csv
from IAMActionHunter.lib.text_formatter import color

import IAMActionHunter.configs.all as configs


def process_cli_args():
    """
    Parse the command-line arguments
    returns: argparse object: command-line arguments
    """
    parser = argparse.ArgumentParser(
        description=(
            "Collect all policies for all users/roles in an AWS account and then query the policies for permissions."
        )
    )
    group = parser.add_mutually_exclusive_group(required=True)
    parser.add_argument(
        "--profile",
        help="The name of the AWS profile to use for authentication for user/role collection.",
    )
    group.add_argument(
        "--account",
        help="Account number to query.",
    )
    parser.add_argument(
        "--query", help="Permissions to query. A string like: s3:GetObject or s3:* or s3:GetObject,s3:PutObject"
    )
    parser.add_argument(
        "--role",
        help="Filter role to query.",
    )
    parser.add_argument(
        "--user",
        help="Filter user to query.",
    )
    parser.add_argument(
        "--all-or-none",
        help="Check if all queried actions are allowed, not just some.",
        action="store_true",
        default=False,
    )
    group.add_argument(
        "--collect",
        help="Collect user and role policies for the account.",
        action="store_true",
        default=False,
    )
    group.add_argument(
        "--list",
        help="List accounts available to query.",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--csv",
        help="File name for CSV report output.",
    )
    parser.add_argument(
        "--config",
        help="JSON config file for preset queries.",
    )
    args = parser.parse_args()

    if args.collect and not args.profile:
        parser.error("--collect requires --profile")
    if args.account and not (args.query or args.config):
        parser.error("--account requires --query or --config")
    return parser.parse_args()


def ensure_list(item):
    """
    Ensure that the item is a list
    Args: item (object): item to check
    returns: list: item as a list
    """
    if isinstance(item, list):
        return item
    else:
        return [item]


def process_cli_query(permissions_json, principal_type_name, cli_args):
    """
    Iterate through all files in the roles directory
    and get the resources for the specified action
    Args: policy_file (string): file name to process
    Args: cli_args (object): command line arguments
    returns: None
    """
    try:
        query_results = {}

        # Get the resources for the queried actions
        query = cli_args.query.split(",")

        # Expand the actions for the query actions to a list
        query = Statement({"Action": query}).actions_expanded

        for action in query:
            if permissions_json.get(action):
                query_results.update({action: permissions_json.get(action)})

        # If the all-or-none flag is set, check if all actions are allowed
        if cli_args.all_or_none:
            if all(item in query_results for item in query):
                pass
            else:
                query_results = {}

        # Write CSV file if specified
        if cli_args.csv:
            process_json_and_append_to_csv(query_results, cli_args.csv, principal_type_name)

        # Print the results to the console
        for action in query_results:
            action_object = query_results[action]
            if action_object["Allow_resources"]:
                print(
                    f"{color.green('[+]')} {color.green(principal_type_name)} can perform {color.green(action)} on the"
                    " following resources:"
                )

                for resource in action_object["Allow_resources"]:
                    print(resource)

                if action_object["Allow_conditions"]:
                    print(color.yellow("[-] With the following conditions:"))
                    for condition in action_object["Allow_conditions"]:
                        print(condition)

                if action_object["Deny_resources"]:
                    print(color.red("[-] If the resources are not included in:"))
                    for resource in action_object["Deny_resources"]:
                        print(resource)
                    if action_object["Deny_conditions"]:
                        print("[-] These Deny rules only apply if the following conditions are met:")
                        for condition in action_object["Deny_conditions"]:
                            print(condition)
                print()

    except Exception as e:
        print()
        print(f"[!!] Error with {principal_type_name}")
        print(e)
        traceback.print_exc()
        print()


def process_config_file_query(permissions_json, config, principal_type_name, cli_args):
    """
    Iterate through all statements for a user/role policy
    and get the resources for the specified action specified in the config file
    output to the console and CSV file if specified
    Args: policy_file (string): file name to process
    Args: cli_args (object): command line arguments
    returns: None
    """
    query_results = {}
    query = config["ActionsNeeded"]

    # Expand the actions for the query actions to a list
    query = Statement({"Action": query}).actions_expanded

    for action in query:
        if permissions_json.get(action):
            query_results.update({action: permissions_json.get(action)})

    # Write CSV file if specified
    if cli_args.csv:
        process_json_and_append_to_csv(query_results, cli_args.csv, principal_type_name)

    if query_results:
        print(f"{color.green('[+]')} {color.green(principal_type_name)} vulnerable to {color.green(config['Name'])}:")
        for action in query_results:
            action_object = query_results[action]
            if action_object["Allow_resources"]:
                print(f"{color.green(action)}")
                for resource in action_object["Allow_resources"]:
                    print(resource)
                if action_object["Allow_conditions"]:
                    print(color.yellow("[-] With the following conditions:"))
                    for condition in action_object["Allow_conditions"]:
                        print(condition)
                if action_object["Deny_resources"]:
                    print(color.red("[-] If the resources are not included in:"))
                    for resource in action_object["Deny_resources"]:
                        print(resource)
                    if action_object["Deny_conditions"]:
                        print("[-] These Deny rules only apply if the following conditions are met:")
                        for condition in action_object["Deny_conditions"]:
                            print(condition)


def get_file_list(directory):
    """
    Get a list of files in a directory
    Args: directory (string): directory to list files
    returns: list: list of files in the directory
    """
    files = [os.path.join(directory, f) for f in os.listdir(directory) if os.path.isfile(os.path.join(directory, f))]
    return files


def filter_files(files, filter):
    """
    Filter a list of files based on a filter
    Args: files (list): list of files to filter
    Args: filter (string): filter to apply to the list of files
    returns: list: list of filtered files
    """
    filtered_files = []
    for file in files:
        if filter.lower() in file.lower():
            filtered_files.append(file)
    return filtered_files


def main():
    args = process_cli_args()

    # If the list flag is set, list all accounts in the output directory
    # and exit
    if args.list:
        for account in os.listdir("actionhunter_output"):
            print(account)
        sys.exit(0)

    output_directory = f"actionhunter_output/{args.account}"

    if args.collect:
        get_all_iam_policies(args.profile)

    else:
        if not os.path.exists(output_directory):
            print(f"{args.account} does not exist. Run with --collect and --profile <aws-profile> first")
            sys.exit(1)

        role_directory = f"{output_directory}/roles"
        user_directory = f"{output_directory}/users"

        # List all files in the roles directory
        role_files = get_file_list(role_directory)

        # List all files in the users directory
        user_files = get_file_list(user_directory)

        all_files = user_files + role_files

        # Filter files based on the role or user specified
        filtered_roles = []
        filtered_users = []
        if args.role:
            for role in args.role.split(","):
                filtered_roles += filter_files(role_files, f"{role}.json")
        if args.user:
            for user in args.user.split(","):
                filtered_users += filter_files(user_files, f"{user}.json")
        if args.role or args.user:
            all_files = filtered_roles + filtered_users

        if not all_files:
            print("[-] No users or roles found for that query")
            sys.exit(1)

        if args.config:
            # Try to load config from builtin configs
            if args.config in vars(configs):
                query_config = vars(configs)[args.config]
            else:
                # Else try to load a config file
                try:
                    with open(f"configs/{args.config}.json", "r") as f:
                        query_config = json.loads(f.read())
                except FileNotFoundError:
                    try:
                        with open(args.config, "r") as f:
                            query_config = json.loads(f.read())
                    except FileNotFoundError:
                        print(f"{args.config} does not exist. Please specify a valid config file or name")
                        sys.exit(1)

        # Iterate through all files and process them
        for permission_file in all_files:
            principal_type = permission_file.split("/")[-2][:-1]
            principal_name = permission_file.split("/")[-1].split(".json")[0]

            with open(permission_file, "r") as f:
                permissions = json.load(f)

            if args.config:
                for config in query_config:
                    process_config_file_query(permissions, config, f"{principal_type}:{principal_name}", args)
            else:
                process_cli_query(permissions, f"{principal_type}:{principal_name}", args)


if __name__ == "__main__":
    main()



from policyuniverse.statement import Statement


class ExtendedStatement(Statement):
    """
    Extends the Statement class to add the notresources and conditions properties
    """

    @property
    def notresources(self):
        # If the statement has NotResource, add a notresources attribute to the PU statement object
        if "NotResource" in self.statement:
            if isinstance(self.statement.get("NotResource"), str):
                return set([self.statement.get("NotResource")])
            return set(self.statement.get("NotResource"))
        else:
            return set()

    @property
    def conditions(self):
        # Add a conditions attribute to the PU statement object
        return self.statement.get("Condition") or {}


def convert_sets_to_lists(obj):
    """
    Converts sets to lists in a dictionary
    Args: obj (dict): a dictionary object
    returns: dict: a dictionary object with sets converted to lists
    """
    if isinstance(obj, set):
        return list(obj)
    elif isinstance(obj, dict):
        return {key: convert_sets_to_lists(value) for key, value in obj.items()}
    elif isinstance(obj, list):
        return [convert_sets_to_lists(element) for element in obj]
    else:
        return obj


def enumerate_actions_resources_for_statements(list_of_statements):
    """
    Enumerates resources and expanded actions for a list of statements
    Args: list_of_statements (list): list of statement JSON objects from a policy
    returns: dict: a dictionary of actions with the following structure:

    {
        "someAction": {
            "Deny_resources": [],
            "Deny_conditions": [],
            "Allow_resources": [],
            "Allow_conditions": []
        },
        "someOtherAction": {
            "Deny_resources": [],
            "Deny_conditions": [],
            "Allow_resources": [],
            "Allow_conditions": []
        }
    }
    """
    results = {}

    def new_action_dict():
        # returns a new action dictionary
        return {
            "Deny_resources": set(),
            "Deny_conditions": [],
            "Allow_resources": set(),
            "Allow_conditions": [],
        }

    # actions_to_check = []
    for st in list_of_statements:
        try:
            statement = ExtendedStatement(st)
        except Exception as e:
            print(e)
            print("[!] Error parsing statement")
            continue

        # Expand the actions to check using policyuniverse
        # actions_to_check = query_actions  # Statement({"Action": query_actions}).actions_expanded

        # Get all the query actions which are in the statement
        # found_actions = [action for action in actions_to_check if action in statement.actions_expanded]

        # iterate through the found query actions
        for action in statement.actions_expanded:
            effect_key = statement.effect

            # Set the action dictionary to the results dictionary if it exists
            # otherwise create a new action dictionary
            action_dict = results.get(action, new_action_dict())

            # Add resources to the Deny_resources set if there is a notresources and effect is Allow
            if statement.notresources and statement.effect == "Allow":
                updated_resources = action_dict["Deny_resources"].union(statement.notresources)
                action_dict.update({"Deny_resources": updated_resources})

            if statement.notresources and statement.effect == "Deny":
                # Add a condition in this case since it means access is denied
                # but does not mean any other access is allowed
                # TODO maybe a better way to do this but for now here we are.
                action_dict["Deny_conditions"].append({"IfResourcesNotIn": statement.notresources})

            if statement.notresources and statement.effect == "Allow":
                # Add a condition in this case since it means access is allowed
                # to everything except the notresources
                # TODO maybe a better way to do this but for now here we are.
                action_dict["Allow_conditions"].append({"IfResourcesNotIn": statement.notresources})

            # Update the Allow or Deny resources
            updated_resources = action_dict[f"{effect_key}_resources"].union(statement.resources)
            action_dict.update({f"{effect_key}_resources": updated_resources})

            # Add conditions if any exist
            if statement.conditions:
                action_dict[f"{effect_key}_conditions"].append(statement.conditions)

            # Update the results for the actions
            results[action] = action_dict

    # if all_or_none_actions and not all(
    #     results.get(action, {"Allow_resources": {}})["Allow_resources"]
    #     for action in actions_to_check
    # ):
    #     # If all_or_none_actions is True, check if all the query actions are in the results
    #     # If not, return an empty dictionary
    #     results = {}

    return convert_sets_to_lists(results)



import pandas as pd
import os


def process_json_and_append_to_csv(json_data, csv_file, principal):
    """
    Process the JSON data and append it to the CSV file
    Args: json_data (string): JSON data to process
    Args: csv_file (string): CSV file to append to
    returns: None
    """

    # Load JSON data into a Python dictionary
    # data = json.loads(json_data)

    # Create an empty DataFrame with the required columns
    df = pd.DataFrame(
        columns=[
            "Principal",
            "Allowed Action",
            "Denied Action",
            "Resource",
            "Conditions",
        ]
    )

    # Create a list to store DataFrame rows
    rows = []

    # Populate the DataFrame with the JSON data
    for action in json_data:
        for allowed_resource in json_data[action]["Allow_resources"]:
            rows.append(
                {
                    "Principal": principal,
                    "Allowed Action": action,
                    "Denied Action": "",
                    "Resource": allowed_resource,
                    "Conditions": json_data[action]["Allow_conditions"],
                }
            )

        for denied_resource in json_data[action]["Deny_resources"]:
            rows.append(
                {
                    "Principal": principal,
                    "Allowed Action": "",
                    "Denied Action": action,
                    "Resource": denied_resource,
                    "Conditions": json_data[action]["Deny_conditions"],
                }
            )

    # Concatenate the rows to the DataFrame
    df = pd.concat([df, pd.DataFrame(rows)], ignore_index=True)

    # Check if the CSV file exists, and write the header if necessary
    if not os.path.isfile(csv_file):
        df.to_csv(csv_file, index=False, mode="w", header=True)
    else:
        df.to_csv(csv_file, index=False, mode="a", header=False)



from colorama import Fore, Style


class color:
    """
    Colorama color class
    Usage: print(color.red("This is red text"))
    Args: string (str): String to color
    """

    def red(string):
        return f"{Fore.RED}{string}{Style.RESET_ALL}"

    def green(string):
        return f"{Fore.GREEN}{string}{Style.RESET_ALL}"

    def yellow(string):
        return f"{Fore.YELLOW}{string}{Style.RESET_ALL}"
